###################################################
# CommentConvert - Converts comments to html file #
###################################################
import sys; sys.dont_write_bytecode = True
import argparse
import json

from commentModels import ScrappedComment
from convertor import HtmlConvertor

VERSION = "0.1.0"


def U_printConversionHelp():
    print("When converting JSON to HTML there are several ways of doing so")
    print("(0) - Basic - All comments in JSON are extracted to single HTML - has performance issues with large ammount of comments")
#    print("(1) - Paged - Comments are extracted " + convertor.MAX_COMMENTS_PER_PAGE + " to separate html files which are then linked")
    print("(2) - PagedDynamic - Just like (1) but pages are loaded using iframes so video will stay loaded - may have compatibility issues with older browsers")
    print("(3) - DynamicalyLoaded - Loads comments using javascript - Requires javascript - so compatibility issues may come")


# Will be used later to a decide whether we want to write universalcomment json or just html json
# Currently unused since universal comment is not implemented yet
def U_isOutputHtml(out):
    """Automatically determines whether we want implicit --output-html or  --output-json"""
    if not out:
        print("Some output file required!")
        sys.exit(1)
    if out.endswith("html"):
        return True
    elif out.endswith("json"):
        return False
    else:
        print("Cannot determine file using extension! Either use (--output-html or --output-json) or the correct .extension")
        sys.exit(1)


def __parseArgs(args):
    # The unimplemented options are mostly commented out to not waste space in help message
    parser = argparse.ArgumentParser(description="Converts comments json file into universal comment and/or html\n Ver: " + VERSION)
    parser.add_argument("--input", "-i", type=argparse.FileType("r"), required=True,
                        help="Input JSON with comments. Type will be resolved automatically but please specify type with '-a' or '-s' if possible")
    parser.add_argument("--api", "-a", action="store_true", help="Given input file is NOT scrapped by youtube_comment_downloader but from youtube API")
    parser.add_argument("--output", "-o", type=str, help="Output file, whether its universal comment JSON or HTML is decided by extension. Default is [input].html")
#    parser.add_argument("--output-html", "-oh", type=str, help="Output file for writing comments converted to HTML page")
#    parser.add_argument("--output-json", "-oj", type=str, help="Output file for writing universal comment JSON")
    parser.add_argument("--videos", "-v", type=str, nargs="+", default=list(),  # Not sure if we should verify existence of videos as well
                        help="Path to the video(s) that should be linked the resulting HTML file, first having the highest load priority")
    parser.add_argument("--scripts", "-s", type=str, nargs="+", default=list(),
                        help="Path to the JS script(s) that should be linked the resulting HTML file - replaces defaults")
    parser.add_argument("--styles", "-c", type=str, nargs="+", default=list(),
                        help="Path to the CSS style(s) that should be linked the resulting HTML file - replaces defaults")
    parser.add_argument("--no-link-default", "-nld", action="store_true", dest="noLinkDefault",
                        help="Wont link default JS and CSS thus producing clean html")
#    parser.add_argument("--download-avatars", "-da", type=str, help="Path to directory where avatars should be downlaoded. If ommited, no avatars are downlaoded")
#    parser.add_argument("--html-conversion-mode", "-hcm", type=int,
#                        help="Number 0-3 Specifies how should the resulting HTML should look (default 0), for more info check --conversion-help or README.md")
#    parser.add_argument("--conversion-help", action="store_true", help="Prints additional information for conversion between JSON and html")

    return parser.parse_args()


def smartCommentParse(reader):
    """Parses comments in more advanced way to pair replies to their parent"""
    print("Parsing JSON...")
    commentList = list()
    line = reader.readline()
    lastRoot = 0
    lineCount = 0  # Maybe we could display number of coments in resulting html?

    while not line == "":
        sys.stdout.write("Parsing line: " + str(lineCount) + "\r")
        comment = ScrappedComment(json.loads(line))

        if comment.isReply:  # we have to find parent to this comment
            if comment.getCid() == commentList[lastRoot].getCid():  # Replies are often comming right after each other
                commentList[lastRoot].addReply(comment)             # so we can check if we haven't found it already
                continue                                            # and save ourselves the iterating

            for i in range(0, len(commentList)):  # for parent to this comment
                if commentList[i].getCid() == comment.getParent():
                    commentList[i].addReply(comment)  # I Wonder what will happen if we wont find it
                    lastRoot = i
                    break
            # This was originally here to check if we went entire loop without figuring out where the parent is but it's actually not that easy to catch it
            # So if problems would arise here is error template i guess. But at this point it prints everytime we pass reply
            # print("WARNING:<smartCommentParse>: This should not happen! We have reply without parent somewhere in commentList.")
            # print("index is somewhere around: " + str(len(commentList) - 1) + " (Actual line is: " + str(lineCount) + " cid: " + comment.getRawCid() + ")")
        else:
            commentList.append(comment)
        line = reader.readline()
        lineCount += 1
    reader.close()
    return commentList


def main(argv=None):
    args = __parseArgs(argv)

    if args.api:
        print("API comments are not supported yet")
        sys.exit(1)

    comlist = smartCommentParse(args.input)  # We are sure it's not api file now
    conv = HtmlConvertor(comlist)

    if not args.noLinkDefault:
        # We can append as argparser should always initialize the lists for us
        # Prepending as we probably want additional files to have higher priority
        args.styles.insert(0, "style.css")
        args.scripts.insert(0, "script.js")

    conv.videos = args.videos
    conv.scripts = args.scripts
    conv.styles = args.styles

    # BUG: This will cut out last letter if the input has no "." in name
    out = args.output if args.output else args.input.name[:args.input.name.rfind(".")] + ".html"

    conv.export(out)
    print("All jobs done, exiting...")


if __name__ == "__main__":
    main(sys.argv[1:])
