##########################################
# Convertor - Converts JSON to HTML page #
##########################################

import commentModels

# Not sure if these "constants" should be inside the class
# Everything that has "U" prefix is Unimplemented and will be ignored when passed
COMMENT_PAGING_STYLE_NONE = -1  # Nothing, just dumps everything to single html
U_COMMENT_PAGING_STYLE_LINK = 0  # DEFAULT Links to another html file after every MAX_COMMENTS_PER_PAGE comment count, user then clicks to visit more comments
U_COMMENT_PAGING_STYLE_JAVASCRIPT = 2  # EXPERIMENTAL Loads comments dynamically from JSON - this may not work in chrome as chrome (alegedly) does not allow javascript to read from local filesystem
U_COMMENT_PAGING_STYLE_IFRAME = 1  # EXPERIMENTAL Loads more comments by embeding another html into current html once user scrolls enough - browser must support this

U_AVATAR_LINK_ONLY = 1  # EXPERIMENTAL Only links avatars using their current link; not sure why would you wanted to do that
AVATAR_DOWNLOAD = 0  # DEFAULT - downloads avatars to their dir and links them with html
AVATAR_NONE = -1  # Will NOT download or link anything

COMMENT_DISPLAY_STYLE_UNPACKED = 0  # DEFAULT Everything is expanded, may be a bit messy when theres to many replies
# Ironically both of those are somehow implemented now for easier debug, however the JS and UNPACKED could be technically merged as they can coexist
U_COMMENT_DISPLAY_STYLE_SCRIPTED = 1  # EXPERIMENTAL requires javascript, you will be able to display and hide replies like on youtube
U_COMMENT_DISPLAY_STYLE_IMAGEBOARD = 2  # EXPERIMENTAL replies will be display in on-hover like fashion; like you can see on most image boards

# Contains only he stackable parts that actually link stuff, if theres more needed (like for <video>) thats not inlcuded
_VIDEO_LINK_TEMPLATE = '<source src="{source}" type="{type}">\n'
_SCRIPT_LINK_TEMPLATE = '<script src="{source}"></script>\n'
_STYLE_LINK_TEMPLATE = '<link rel="stylesheet" type="text/css" href="{source}"/>\n'


# Decided for class because otherwise we would have to call function with giant ammount of parametters
class HtmlConvertor:

    # Some global config
    U_MAX_COMMENTS_PER_PAGE = 1000  # Todo: Test this, mainly with firefox chrome seems to be able to handle anything
    COMMENT_PAGING_STYLE = 0
#    AVATAR_ACTION = -1
    COMMENT_DISPLAY_STYLE = 0

    comments: list
#    avatarDir: str
    videos: list
    styles: list
    scripts: list

    showProgress: bool = True  # Not sure if it will be even needed as the operations look too fast to even display %

    # Constructor bude mít jen to bez čeho se absolutně nedá obejít, takže JSON a output file, zbytek je teoreticky optional
    def __init__(self, comments):
        self.comments = comments

    def export(self, outputFile):
        """Exports the resulting html, applying properties of this class as modifications"""
        print("Exporting to HTML...")
        writer = open(outputFile, "w")
        self.__writeHead(writer)
        self.__writeVideoLink(writer)
        self.__writeComments(writer)
        self.__finalize(writer)
        writer.close()

    # Writes head and links html with CSS
    def __writeHead(self, writer):
        # Not sure if there is any better way of saving these strings but this looks awful
        headStart = """\
<!DOCTYPE html>
<html>
    <head>
        <meta charset="UTF-8">
        <title>{VideoName}</title>
"""
        headEnd = """\
    </head>
    <body onLoad="autoLoad()">
"""
        writer.write(headStart)
        try:
            for style in self.styles:
                writer.write(_STYLE_LINK_TEMPLATE.format(source=style))
        except NameError:
            print("No styles linked")
        writer.write(headEnd)

    # Change to general linking tht will link js, css, video
    def __writeVideoLink(self, writer):
        """Writes video html part to provided writer"""
        videoLinkOpen = "<video controls>\n"
        videoLinkClose = "\nYour browser does not support video playing.\n</video>"
        writer.write(videoLinkOpen)
        try:
            for videoFile in self.videos:
                writer.write(_VIDEO_LINK_TEMPLATE.format(source=videoFile, type=self.__getVideoType(videoFile)))
        except NameError:
            print("No video files specified, not wiriting any videos")
        writer.write(videoLinkClose)

    def __writeComments(self, writer):
        """Decides which writing method to use and writes comments to html"""  # todo split this method to multiple ones that will handle diffrent comment writing types
        openComments = '<ul class="comment-secion">\n'
        writer.write(openComments)

        for com in self.comments:
            self.__writeSingleComment(com, writer)

        writer.write("</ul>\n")

    def __writeSingleComment(self, comment, writer):
        writer.write('<li id="' + comment.getCid() + '">\n')  # We have to generate unique ID for reply ul that will be associated with parent
        writer.write('<p class="comment-author"><a href="' + comment.getChannelLink() + '">' + comment.author + '</a> (<em class="comment-author-cid">' + comment.getCid() + '</em>)</p>\n')  # Todo: Link channel?
        writer.write('<p class="comment-text">' + comment.text + '</p>\n')  # Technically we dont need to write replying string as it should never be reply here
        if comment.hasReplies():  # We write all of it's children
            # Inject button here
            writer.write('<button class="show-reply-chain" type="button" onClick="toggleReplies(\'' + comment.getCid() + '\')">Toggle ' + str(len(comment.replies)) + ' replies</button>\n')
            writer.write('<ul class="comment-reply-chain" id="' + comment.getCid() + '-children">\n')
            for reply in comment.replies:  # Todo: We have to check for \ns so we have correct new lines
                writer.write('<li id="' + reply.getCid() + '" class="comment-reply">\n')
                writer.write('<p class="comment-author">' + reply.author + ' (<em class="comment-author-cid">' + reply.getCid() + '</em>)</p>\n')  # Todo: Link channel?
                writer.write('<p class="comment-quote"><a href=#' + reply.getParent() + '> >> ' + reply.getParent() + '</a></p>\n')
                writer.write('<p class="comment-text">' + reply.text + '</p>\n')
                writer.write('</li>\n')
            writer.write("</ul>\n")
        writer.write("</li>\n")

    def __finalize(self, writer):
        """Finishes writing html and links every JS file passed to this class"""
        try:
            for script in self.scripts:
                writer.write(_SCRIPT_LINK_TEMPLATE.format(source=script))
        except NameError:  # Should never happen, list should be at least empty
            print("No javascript linked")
        writer.write("</body>\n</html>\n")
        writer.flush()

    def __getVideoType(self, path):
        """Checks whether video type is supported and returns html indentifier of it's format"""
        if path.endswith("mp4") or path.endswith("ogg") or path.endswith("webm"):
            return "video/" + path[path.rfind(".") + 1: len(path)]
        else:
            print("Unsupported video: " + path + " Setting type to mp4")  # Todo: anything else we could do here? make this calling point to call another script from?
            return "video/mp4"
